/**
  * @file ThermostatAVR.c
  * @brief This module is used to control a temperature in HEATING process 
           and displays it on LCD
  *
  * @author Stefan Popovic
  *
  * @date 29/11/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: ATMEGA32
  * Dev.Board: EasyAVR v7 Connectivity
  * Oscillator: 8 MHz external
  * Ext. Modules: Thermostat Click Board
  * SW: MikroC PRO for AVR v6.1.2

  * NOTES:
    - Place Thermostat click board on mikroBUS slot 1
    - Turn on TWI switches SW6.3 and SW6.4. (board specific)
    - Turn on the LCD backlight on the EasyAVR v7 board (SW3.1)
    - Set referent temperature via Set_TEMP_REF routine
    - Set temperature hysteresis via Set_TEMP_HYST routine
    - TRES: PA6
    - TE:   PA5
  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/
  #define HYST_TEMP     18
  #define REF_TEMP      23
  #define SLAVE_ADDR  0x90                        // Slave address

  #define TEMP_REG    0x00
  #define CONFIG_REG  0x01
  #define HYST_REG    0x02
  #define OS_REG      0x03
  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/


  /**************************************************************************
  * Module Typedefs
  **************************************************************************/


  /**************************************************************************
  * Includes
  **************************************************************************/
  #include "built_in.h"
  #include "recources.h"
  #include <stdint.h>
  #include <stdbool.h>

  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/
  sbit TRES  at PORTA6_bit;
  sbit TE    at PORTA5_bit;
  
  char txt[10];
  char txt1[] = "ROOM TEMP";
  char txt2[] = "RELAY ON";
  char txt3[] = "C";
  
  // LCD module connections
  sbit LCD_RS at PORTA2_bit;
  sbit LCD_EN at PORTD6_bit;
  sbit LCD_D4 at PORTC4_bit;
  sbit LCD_D5 at PORTC5_bit;
  sbit LCD_D6 at PORTC6_bit;
  sbit LCD_D7 at PORTC7_bit;

  sbit LCD_RS_Direction at DDA2_bit;
  sbit LCD_EN_Direction at DDD6_bit;
  sbit LCD_D4_Direction at DDC4_bit;
  sbit LCD_D5_Direction at DDC5_bit;
  sbit LCD_D6_Direction at DDC6_bit;
  sbit LCD_D7_Direction at DDC7_bit;
  // End LCD module connections

    void Display_Init()
  {
      Lcd_Init();                                 // Initialize LCD
      Lcd_Cmd(_LCD_CLEAR);                        // Clear display
      Lcd_Cmd(_LCD_CURSOR_OFF);                   // Cursor off
      Lcd_Out(1, 1, txt1);
      Lcd_Out(2, 1, txt2);
      Lcd_Out(1, 16, txt3);
      Lcd_Out(2, 16, txt3);
   }

  
  // Variables
  unsigned short temp_MSB;                        // Store temperature register MSB byte
  unsigned short temp_LSB;                        // Store temperature register LSB byte

  unsigned short mask_0;                          // Mask for display 0
  unsigned short mask_1;                          // Mask for display 1
  unsigned short mask_2;                          // Mask for display 2
  unsigned short mask_3;                          // Mask for display 3

  void Temp_Read(void);
  void Temp_display(void);
  void Display_TEMP(void);
  void Set_TEMP_REF(unsigned short);
  void Set_TEMP_HYST(unsigned short);
  void Set_CONFIG(unsigned short);
  
  // Main program
  void main() {
       // MCU Init
       DDRA  = 0xFF;                              // Configure PORTA as output
       DDRC  = 0xFF;                              // Configure PORTC as output
       DDRD  = 0xFF;                              // Configure PORTD as output
       
       PORTA = 0x00;                              // Initial PORTA value
       PORTC = 0x00;                              // Initial PORTC value
       PORTD = 0x00;                              // Initial PORTD value

       // Lcd Init
       Display_Init();
       
       // TWI Init
       TWI_Init(100000);

       //***********************************************************************

       // MAX7502 temperature controller IC

       // MAX7502 Initialization
       TE = 0;                   // Thermostat function disabled - Output Relay disconnected
       TRES = 0;                 // Apply Reset procedure
       delay_ms(150);
       TRES = 1;                 // Temperature controller is in default state and ready for work

       // MAX7502 Configuration
       Set_CONFIG(0);            // Set parameters in configuration register
                                 // Refer to MAX7502 datasheet and click board schematic in order to
                                 // Set right parameters for intended application:
                                 // E.g. set OS output pin polarity for using the chip sensor in
                                 // Processes of Heating (bit1=0) or Cooling (bit1=1)
                                 // Or set the output function of the same pin to be Comparator (bit2=0)
                                 // Or Interrupt logic (bit2=1) in order to use the sensor for hysterezis
                                 // Temp ON/OFF control (real process) or trivial ON/OFF control
       // In this example the chip is configured for heating
       Set_TEMP_REF(REF_TEMP);   // Set referent temperature T = 24 C (Output relay is ON until the environment
                                 // temperature has reach that value)
       Set_TEMP_HYST(HYST_TEMP); // Output Relay is OFF until the temperature falls below HYST value

       TE = 1;                   // Thermostat enabled - Output Relay connected to OS output pin of MAX7502
       
       //***********************************************************************
       ShortToStr(HYST_TEMP, txt);
       Lcd_Out(2, 11, txt);

       //***********************************************************************
       // Temperature read loop  -  temperature controlling process
       while (1) 
       {
             Temp_read();                         // Read 9-bit temperature value from MAX7502's 
                                                  //    register space
                                                  // +-0.5 C fraction neglected in this example
             delay_ms(500);
             
             // Show temperature on the LCD
             ShortToStr(temp_MSB, txt);
             Lcd_Out(1, 11, "   ");
             Lcd_Out(1, 11, txt);
       }
       //***********************************************************************
  }

  void Temp_read() {
       TWI_Start();                               // TWI START condition
       TWI_Write(SLAVE_ADDR);                     // Send slave address (RD/W = 0)
       TWI_Write(TEMP_REG);                       // Write pointer (temperature read only register at address 00h)
       TWI_Start();                               // Repeated start
       TWI_Write(SLAVE_ADDR | 0x01);              // Send slave address (RD/W = 1)
       temp_MSB = TWI_Read(1);                    // Read temp MSB byte, send acknowledge
       temp_LSB = TWI_Read(0);                    // Read temp LSB byte, send not acknowledge
       TWI_Stop();                                // I2C STOP condition
  }

  void Set_TEMP_REF(unsigned short _temp) {
       // write byte
       TWI_Start();                               // I2C START condition
       TWI_Write(SLAVE_ADDR);                     // Send slave address (RD/W = 0)
       TWI_Write(OS_REG);                         // Set pointer
       TWI_Write(_temp);                          // Set referent temperature
       TWI_Stop();                                // I2C STOP condition
  }

  void Set_TEMP_HYST(unsigned short _temp) {
       // write byte
       TWI_Start();                               // I2C START condition
       TWI_Write(SLAVE_ADDR);                     // Send slave address (RD/W = 0)
       TWI_Write(HYST_REG);                       // Set pointer
       TWI_Write(_temp);                          // Set temperature hysterezis
       TWI_Stop();                                // I2C STOP condition
  }

  void Set_CONFIG(unsigned short _config_value) {
       // write byte
       TWI_Start();                               // I2C START condition
       TWI_Write(SLAVE_ADDR);                     // Send slave address (RD/W = 0)
       TWI_Write(CONFIG_REG);                     // Set pointer
       TWI_Write(_config_value);                  // Set temperature hysterezis
       TWI_Stop();                                // I2C STOP condition
  }